require "Apollo"
require "GameLib"
require "Window"

if not STARTUP_FUNC then
	STARTUP_LIST = {}
	function STARTUP_FUNC(m)
		local t = GameLib.GetTickCount()
		table.insert(STARTUP_LIST, string.format("%s [%d]",m, t))
	end
end

STARTUP_FUNC("Speedo:Top of Speedo.lua")

local function MergeTable(src,trg)
	for k,v in pairs(src) do
		if type(v) == "table" then
			if trg[k] == nil then
				trg[k] = {}
			end
	 	 	if type(trg[k]) == 'table' then
	 			MergeTable(src[k], trg[k])
	 	 	end
		else
			if trg[k] == nil then
				trg[k] = v
			end
		end
	end
end

local defaults = {
	instant = true
}

local Speedo = {}

gSPEEDO = Speedo

Speedo.count=0
Speedo.ema=0
Speedo.lastupd=0
Speedo.active=false

Speedo.SAVEDVARS = {}
Speedo.active = false

function Speedo:new(o)
	STARTUP_FUNC("Speedo:new")
    o = o or {}
    setmetatable(o, self)
    self.__index = self

    -- initialize variables here

    return o
end

function Speedo:Init()
	STARTUP_FUNC("Speedo.Init")
	local bHasConfigureFunction = false
	local strConfigureButtonText = "Speedo"
	local tDependencies = {
		-- "UnitOrPackageName",
	}

    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

function Speedo:OnLoad()
	STARTUP_FUNC("Speedo:OnLoad")
	self.xmlDoc = XmlDoc.CreateFromFile("Speedo.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	Apollo.RegisterSlashCommand("speedo", "SlashCommand", self)
	Apollo.RegisterSlashCommand("spdadd", "spdSlashCommand", self)
end

function Speedo:spdSlashCommand(h, args)
	Print("+spdSlash")
	local tAddons = GetAddons()
	Print("-spdSlash")
end

function Speedo:SlashCommand(h, args)
	if Speedo.active then
		Print("Speed test end")
		Speedo.active = false
		local endtime = GameLib.GetTickCount()
		local endpos = GameLib.GetPlayerUnit():GetPosition()
		
		local nTime = (endtime - Speedo.actstart)/1000
		local dx = Speedo.actpos.x - endpos.x
		local dy = Speedo.actpos.y - endpos.y
		local dz = Speedo.actpos.z - endpos.z
		
		local dist = math.sqrt((dx*dx)+(dy*dy)+(dz*dz))
		
		Print(string.format("Time: %.2fs, Distance: %.2fm, Speed=%.2fm/s",nTime, dist, (dist/nTime)))
	else
		Print("Speed test start")
		Speedo.active = true
		Speedo.actstart = GameLib.GetTickCount()
		Speedo.actpos = GameLib.GetPlayerUnit():GetPosition()
	end
end

function Speedo:OnDocLoaded()
	STARTUP_FUNC("Speedo:OnDocLoaded")
	if self.xmlDoc and self.xmlDoc:IsLoaded() then
		self.wndSpeedo = Apollo.LoadForm(self.xmlDoc, "Speed", nil, self)
		if self.wndSpeedo == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		if Speedo.SAVEDVARS and Speedo.SAVEDVARS.nOL then
			self.wndSpeedo:SetAnchorOffsets(Speedo.SAVEDVARS.nOL, Speedo.SAVEDVARS.nOT, Speedo.SAVEDVARS.nOR, Speedo.SAVEDVARS.nOB)
		else
			Speedo.SAVEDVARS.nOL, Speedo.SAVEDVARS.nOT, Speedo.SAVEDVARS.nOR, Speedo.SAVEDVARS.nOB = self.wndSpeedo:GetAnchorOffsets()
		end
		self.wndSpeedo:Show(true, true)
		self.currenttick = GameLib.GetTickCount()
		self.playerpos = nil
		self.timer = ApolloTimer.Create(0.05, true, "OnTimer", self)
	end
end

function Speedo:MouseButtonDown(wndHandler, wndControl, eMouseButton)
	if eMouseButton == GameLib.CodeEnumInputMouse.Right then
		Speedo.SAVEDVARS.instant = not Speedo.SAVEDVARS.instant
	end
end

function Speedo:WindowMove()
	Print("Speedo:WindowMove()")
	Speedo.SAVEDVARS.nOL, Speedo.SAVEDVARS.nOT, Speedo.SAVEDVARS.nOR, Speedo.SAVEDVARS.nOB = self.wndSpeedo:GetAnchorOffsets()
end

function Speedo:OnTimer()
	local nTick = GameLib.GetTickCount()
	local pUnit = GameLib.GetPlayerUnit()
	
	local sSpeedoC = "0.0"
	local sSpeedoI = "0.0"
	
	if pUnit then
		local pPos = pUnit:GetPosition()		
		if self.playerpos then	
			local dZ = self.playerpos.z - pPos.z
			local dY = self.playerpos.y - pPos.y
			local dX = self.playerpos.x - pPos.x
			
			local nDist = math.sqrt((dZ*dZ)+(dY*dY)+(dX*dX))
			local nTime = nTick - self.currenttick

			if nTime > 0 and nDist > 0.075 then
				local spd = 1000*nDist/nTime
				local cnt = Speedo.count
				local ema = Speedo.ema
				
				if Speedo.count == 0 then
					ema = spd
					Speedo.count = 1
					Speedo.lastupd = 0
				end
				
				local nema = ema + (0.9*(spd - ema))
				Speedo.ema = nema
				
				sSpeedoC = string.format("%.1f", nema)
				sSpeedoI = string.format("%.1f", spd)
				
			else
				Speedo.count = 0
				Speedo.lastupd = 0
			end
		else
			self.playerpos = {}
		end
		self.currenttick = nTick
		self.playerpos.x = pPos.x
		self.playerpos.y = pPos.y
		self.playerpos.z = pPos.z
	end
	
	if nTick - Speedo.lastupd > 250 then
		Speedo.smooth = sSpeedoC
		Speedo.lastupd = nTick
	end

	if Speedo.SAVEDVARS and Speedo.SAVEDVARS.instant then
		self.wndSpeedo:SetText(string.format("%s [%s] m/s", Speedo.smooth, sSpeedoI))
	else
		self.wndSpeedo:SetText(string.format("%s m/s", Speedo.smooth))
	end
end

function Speedo:OnSave(eType)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		--Speedo.SAVEDVARS.nOL, Speedo.SAVEDVARS.nOT, Speedo.SAVEDVARS.nOR, Speedo.SAVEDVARS.nOB = self.wndSpeedo:GetAnchorOffsets()
		return Speedo.SAVEDVARS
	end
end

function Speedo:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		MergeTable(tSavedData,Speedo.SAVEDVARS)
		MergeTable(defaults,Speedo.SAVEDVARS)
		if self.wndSpeedo and Speedo.SAVEDVARS and Speedo.SAVEDVARS.nOL then
			self.wndSpeedo:SetAnchorOffsets(Speedo.SAVEDVARS.nOL, Speedo.SAVEDVARS.nOT, Speedo.SAVEDVARS.nOR, Speedo.SAVEDVARS.nOB)
		end		
	end
end

-----------------------------------------------------------------------------------------------
-- ItemPreview Instance
-----------------------------------------------------------------------------------------------
local SpeedoInst = Speedo:new()
SpeedoInst:Init()

STARTUP_FUNC("Speedo:End of Speedo.lua")
